/*
 * Copyright (C) 2016 The Dagger Authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package test.optional;

import static com.google.common.truth.Truth.assertThat;
import static test.optional.OptionalBindingComponents.Value.QUALIFIED_VALUE;
import static test.optional.OptionalBindingComponents.Value.VALUE;

import com.google.common.collect.ImmutableList;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;
import test.optional.OptionalBindingComponents.OptionalBindingComponent;

/** Tests for present optional bindings. */
@RunWith(Parameterized.class)
public final class OptionalBindingComponentsPresentTest {

  @Parameters(name = "{0}")
  public static Iterable<Object[]> parameters() {
    return ImmutableList.copyOf(
        new Object[][] {
          {DaggerOptionalBindingComponents_PresentOptionalBindingComponent.create()},
          {DaggerOptionalBindingComponents_AbsentOptionalBindingComponent.create().presentChild()},
        });
  }

  private final OptionalBindingComponent component;

  public OptionalBindingComponentsPresentTest(OptionalBindingComponent component) {
    this.component = component;
  }

  @Test
  public void optionalProvider() {
    assertThat(component.optionalProvider().get().get()).isEqualTo(VALUE);
  }

  @Test
  public void optionalLazy() {
    assertThat(component.optionalLazy().get().get()).isEqualTo(VALUE);
  }

  @Test
  public void optionalLazyProvider() {
    assertThat(component.optionalLazyProvider().get().get().get()).isEqualTo(VALUE);
  }

  @Test
  public void qualifiedOptional() {
    assertThat(component.qualifiedOptionalInstance()).hasValue(QUALIFIED_VALUE);
  }

  @Test
  public void qualifiedOptionalProvider() {
    assertThat(component.qualifiedOptionalProvider().get().get()).isEqualTo(QUALIFIED_VALUE);
  }

  @Test
  public void qualifiedOptionalLazy() {
    assertThat(component.qualifiedOptionalLazy().get().get()).isEqualTo(QUALIFIED_VALUE);
  }

  @Test
  public void qualifiedOptionalLazyProvider() {
    assertThat(component.qualifiedOptionalLazyProvider().get().get().get())
        .isEqualTo(QUALIFIED_VALUE);
  }
}
